#Add the music library
add_library ('minim')

# you must wright here all the global variables
player = None
minim = None
meta = None
fft = None 
beat = None

eRadius = 100
kickSize = 0
snareSize = 0
hatSize = 0

def setup():
    size (1024, 576)
    global minim, player, meta, fft, beat
    
    #always start Minim first!
    minim = Minim(this)
    
    #add jbject for play and load the file with the standart buffer size of 1024 samples
    #player = minim.loadFile ("sweet.mp3")
    #player = minim.loadFile ("celticrock.mp3")
    #player = minim.loadFile ("thanks.mp3")
    player = minim.loadFile ("surrender.mp3")
    
    #get MetaData from file
    meta = player.getMetaData();
    
    """
    an FFT needs to know how 
    long the audio buffers it will be analyzing are
    and also needs to know 
    the sample rate of the audio it is analyzing
    FFT need to have a power of 2 size of the buffer (64, 128, 256, 512, 1024)
    """
    fft = FFT(player.bufferSize(), player.sampleRate())
    
    #averages the FFT output to have a 3 column ptr octave and start from the 40KHz
    fft.logAverages (40,3)
    
    #start playing
    player.play()
    
    #a beat detection object song SOUND_ENERGY mode 
    #with a sensitivity of 10 milliseconds by default
    #beat = BeatDetect()
    
    #a beat detection object song FREQUENCY_ENERGY mode 
    #with a sensitivity of 300 milliseconds
    beat = BeatDetect(player.bufferSize(), player.sampleRate())
    beat.setSensitivity (50);
    
    #some drawing constant and modes
    rectMode (CORNERS)
    
    
def draw():
    demo()
    
def stop():
    global minim, player
    player.close()
    minim.stop()
    
def keyPressed():
    if (player.isPlaying() ):  
         player.pause()
    else : player.play()


###########################################
# waveForm
###########################################
def waveForm():
    """
    we draw the waveform by connecting neighbor values with a line
    we scale each of the values by map fuction 
    because the values in the buffers are normalized
    this means that they have values between -1 and 1. 
    If we don't scale them up our waveform 
    will look more or less like a straight line.
    we also scale the x to fit full screen 
    also using map function (http://py.processing.org/reference/map.html)
    """
    #start loop for all buffer. by default we have 1024 value inside the bufer
    for i in range (player.bufferSize() - 1):
        # count the X position of the line
        # we assume the waveform will be all screen width
        x1 = map (i, 0, player.bufferSize(), 0, width)
        x2 = map (i+1, 0, player.bufferSize(), 0, width)
        # count the Y position for the left and right channel
        # we assume one channel waveform will be a half of the screen height
        # we also plase the "center" of the wave to the 1/4 and 3/4 of the screen height
        left1 = map (player.left.get(i), -1, 1, -1*(height/4), height/4)
        left2 = map (player.left.get(i+1), -1, 1, -1*(height/4), height/4)
        right1 = map (player.right.get(i), -1, 1, -1*(height/4), height/4)
        right2 = map (player.right.get(i+1), -1, 1, -1*(height/4), height/4)
        #drawing the sample line for the left and right channel
        line (x1, height/4+left1, x2, height/4+left2)
        line (x1, 3*height/4+right1, x2, 3*height/4+right2)
        #return to the loop header (for ....) to repeat for the next sample until the last one

###########################################
# textMeta
###########################################        
                
def textMeta(x, y):
    #we get the information from the file and print it to the picture
    #starting in the (x, y) point
    
    # set the text size 
    textSize (15);
    textAlign (LEFT)
    # set value to switch the text line
    yi = 15
    #we will use a meta object from the setup() function
    global meta
    
    text("File Name: " + meta.fileName(), x, y) 
    y = y+yi #increase y to switch to the next line
    text("Length (in milliseconds): " +str ( meta.length()), x, y)
    y = y+yi
    text("Title: " + meta.title(), x, y)
    y = y+yi
    text("Author: " + meta.author(), x, y)
    y = y+yi
    text("Album: " + meta.album(), x, y)
    y = y+yi
    text("Date: " + meta.date(), x, y)
    y = y+yi
    text("Comment: " + meta.comment(), x, y)
    y = y+yi
    text("Track: " + meta.track(), x, y)
    y = y+yi
    text("Genre: " + meta.genre(), x, y)
    y = y+yi
    text("Copyright: " + meta.copyright(), x, y)
    y = y+yi
    text("Disc: " + meta.disc(), x, y)
    y = y+yi
    text("Composer: " + meta.composer(), x, y)
    y = y+yi
    text("Orchestra: " + meta.orchestra(), x, y)
    y = y+yi
    text("Publisher: " + meta.publisher(), x, y)
    y = y+yi
    text("Encoded: " + meta.encoded(), x, y)
    
    y = y+yi+yi
    text("Sample rate: " + str(player.sampleRate()), x, y)
    y = y+yi
    text("Buffer size: " + str(player.bufferSize()), x, y)



###########################################
# position
###########################################
def pos():
    #get the current playing position (time in milisecond)
    #scale it to fit the width of the screen
    #and draw a visual marker 
    
    posx = map (player.position(), 0, player.length(), 0, width)
    
    rect (0, height-20, posx, height)
    line(posx, 0, posx, height)
    
    return posx


###########################################
# spectrum
###########################################
def spectrum():
    """ 
    first perform a forward fft on one of song's buffers
    we are using the mix buffer (left + right)
    but you can use any one you like
    """
    global fft
    fft.forward(player.mix)
 
    #draw the spectrum as a series of vertical lines
    #we also scale the value of i (x) to fit the screen
    #and multiply a value of y to be a little bigger
    
    # average FFT output
    dx = float(width)/fft.avgSize()
    for i in range (fft.avgSize()) :
        rect((i)*dx, height - fft.getBand(i)*4-20, (i+1)*dx, height-20 );
        #output the frequency of the current column when mouse over
        if (mouseX >i*dx) and (mouseX < (i+1)*dx) : 
            curFreq = fft.getAverageCenterFrequency (i)
            text ('%.2f'%curFreq, i*dx, mouseY)   
   
    """
    #pure FFT output!!
    fill (20, 255, 50);
    stroke (20, 255, 50);
    dx = float(width)/fft.specSize()
    text (str(fft.specSize())+"   "+str (width) +"   " + str(dx),100,100)
    for i in range (fft.specSize()) :
        line(i*dx, height-20, i*dx, height - fft.getBand(i)*8-20);
        #output the frequency of the current column when mouse over
        if (mouseX >i*dx) and (mouseX < (i+1)*dx) : 
            curFreq = fft.indexToFreq (i)
            text ('%.2f'%curFreq, i*dx, mouseY+20) 
    """
      
  

###########################################
# beat ellipse SOUND-ENERGY mode
###########################################
def soundEnergy(x, y):
    global eRadius
    beat.detect (player.mix)
    if (beat.isOnset() ) : eRadius = width/4
    ellipse (x, y, eRadius, eRadius)
    eRadius *= 0.95
    
###########################################
# FREQUENCY_ENERGY mode
###########################################
def freqEnergy(x, y):
    global kickSize, snareSize, hatSize
    beat.detect (player.mix)
    
    dx = float(width)/beat.detectSize()
    for i in range (beat.detectSize()) :
        if beat.isOnset(i) :
            rect((i)*dx, 0, (i+1)*dx, height/2 );
        
    
    
    if (beat.isKick() ) : kickSize = height/8 
    #else : kickSize = 0
    if (beat.isSnare() ) : snareSize = height/8 
    #else : snareSize = 0
    if (beat.isHat() ) : hatSize = height/8 
    #else : hatSize = 0
    
    textAlign (CENTER)
    
    textSize (kickSize)
    text ("KICK", width/4, height/2)
    textSize (snareSize)
    text ("SNARE", width/2, height/2)
    textSize (hatSize)
    text ("HAT", 3*width/4, height/2)
    
    kickSize *= 0.95
    snareSize *= 0.95
    hatSize *= 0.95
    
###########################################
# DEMO ALL the function
###########################################    
def demo():    
    stroke (150, 255, 150)
    fill (150, 255, 150)
    textMeta(5, 20) 
    
    if (player.isPlaying()): 
        #remove the BG       
        #background (12, 12, 12, 50)
        fill (12, 12, 12, 20)
        noStroke()
        rect (0, 0, width, height)
        
        fill (150, 200, 255)
        stroke(150, 200, 255)
        waveForm()
    
        fill (150, 150, 150)
        stroke(150, 150, 150)
        pos()
        
        fill (255, 50, 50, 50)
        stroke(255, 50, 50, 50)
        spectrum()
        
        fill (20, 50, 250)
        stroke(20, 50, 250)
        soundEnergy (width/2, height/2)
        
        freqEnergy (1,1)
    
    